package models

import (
	"fmt"
	"hongze/hongze_edb_lib/utils"
	"strconv"
	"strings"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

type BaseFromSmmData struct {
	SmmDataId          int `orm:"column(smm_data_id);pk"`
	BaseFromSmmIndexId int
	IndexCode          string
	DataTime           string
	Value              string
	CreateTime         time.Time
	ModifyTime         time.Time
	DataTimestamp      int64
}

func GetBaseFromSmmDataByCondition(condition string, pars []interface{}) (list []*BaseFromSmmData, err error) {
	o := orm.NewOrm()
	sql := `SELECT * FROM base_from_smm_data WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	_, err = o.Raw(sql, pars).QueryRows(&list)
	return
}

//新增有色指标数据
func AddEdbDataFromSmm(edbCode string) (err error) {
	o := orm.NewOrm()

	var condition string
	var pars []interface{}
	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}
	smmBaseDataAll, err := GetBaseFromSmmDataByCondition(condition, pars)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	var isAdd bool
	addSql := ` INSERT INTO edb_data_ys(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	existMap := make(map[string]string)
	for _, sv := range smmBaseDataAll {
		eDate := sv.DataTime
		dataTime, err := time.Parse(utils.FormatDate, eDate)
		if err != nil {
			return err
		}
		timestamp := dataTime.UnixNano() / 1e6
		timeStr := fmt.Sprintf("%d", timestamp)
		if _, ok := existMap[eDate]; !ok {
			addSql += GetAddSql("0", edbCode, eDate, timeStr, sv.Value)
			isAdd = true
		}
		existMap[eDate] = sv.Value
	}
	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		utils.FileLog.Info("addSql:" + addSql)
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

//刷新有色指标数据
func RefreshEdbDataFromSmm(edbInfoId int, edbCode, startDate string) (err error) {
	source := utils.DATA_SOURCE_YS
	o := orm.NewOrm()
	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)
	//计算数据
	var condition string
	var pars []interface{}

	if edbCode != "" {
		condition += " AND index_code=? "
		pars = append(pars, edbCode)
	}

	if startDate != "" {
		condition += " AND data_time>=? "
		pars = append(pars, startDate)
	}

	smmDataList, err := GetBaseFromSmmDataByCondition(condition, pars)

	var existCondition string
	var existPars []interface{}

	existCondition += " AND edb_info_id=? "
	existPars = append(existPars, edbInfoId)
	if startDate != "" {
		existCondition += " AND data_time>=? "
		existPars = append(existPars, startDate)
	}

	existList, err := GetEdbDataByCondition(source, existCondition, existPars)
	if err != nil {
		return err
	}
	existMap := make(map[string]*EdbInfoSearchData)
	for _, v := range existList {
		existMap[v.DataTime] = v
	}
	addSql := ` INSERT INTO edb_data_ys(edb_info_id,edb_code,data_time,value,create_time,modify_time,data_timestamp) values `
	var isAdd bool
	for _, v := range smmDataList {
		item := v
		if findItem, ok := existMap[v.DataTime]; !ok {
			eDate := item.DataTime
			sValue := item.Value

			dataTime, err := time.Parse(utils.FormatDate, eDate)
			if err != nil {
				return err
			}
			timestamp := dataTime.UnixNano() / 1e6
			timeStr := fmt.Sprintf("%d", timestamp)

			addSql += GetAddSql(edbInfoIdStr, edbCode, eDate, timeStr, sValue)
			isAdd = true
		} else {
			if findItem != nil && utils.SubFloatToString(findItem.Value, 30) != item.Value {
				err = ModifyEdbDataById(source, findItem.EdbDataId, item.Value)
				if err != nil {
					return err
				}
			}
		}
	}

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			fmt.Println("RefreshEdbDataFromSmm add Err", err.Error())
			return
		}
	}
	return
}
